from collections import namedtuple

INITIAL_URL = "https://progenomes.embl.de/data"

URLMapping = namedtuple("URLMapping", ["file_prefix", "filename", "filetype", "headers"])

URL_MAPPING = {
    "habitats-per-isolate": URLMapping(
        file_prefix="proGenomes3",
        filename="habitat_isolates",
        filetype="tab.bz2",
        headers=True,
    ),
    "habitats-per-speci-cluster": URLMapping(
        file_prefix="proGenomes3",
        filename="habitat_specI",
        filetype="tab.bz2",
        headers=True,
    ),
    "representatives-per-speci-cluster": URLMapping(
        file_prefix="proGenomes3",
        filename="representatives_for_each_specI",
        filetype="tsv.gz",
        headers=False,
    ),
    "speci-clustering-data": URLMapping(
        file_prefix="proGenomes3",
        filename="specI_clustering",
        filetype="tab.bz2",
        headers=False,
    ),
    "gtdb-taxonomy": URLMapping(
        file_prefix="proGenomes3",
        filename="specI_lineageGTDB",
        filetype="tab.bz2",
        headers=True,
    ),
    "highly-important-strains": URLMapping(
        file_prefix=None,
        filename="highly_important_strains",
        filetype="tab.bz2",
        headers=False,
    ),
    "mge-orfs": URLMapping(
        file_prefix="representatives",
        filename="mge_ORFS",
        filetype="tsv.bz2",
        headers=True,
    ),
    "mge-annotation": URLMapping(
        file_prefix="representatives",
        filename="mge_annotation",
        filetype="tsv.bz2",
        headers=True,
    ),
}


def get_url(item: str):
    try:
        mapping = URL_MAPPING[item]
    except KeyError as exc:
        raise ValueError(f"Item '{item}' not found in URL mapping.") from exc
    if mapping.file_prefix is None:
        path = f"{mapping.filename}.{mapping.filetype}"
    else:
        path = f"{mapping.file_prefix}_{mapping.filename}.{mapping.filetype}"
    return (f"{INITIAL_URL}/{path}", mapping.filetype)


def view(target):
    import polars as pl
    import pandas as pd
    url, filetype = get_url(target)
    if "tab.bz2" in filetype:
        return pl.from_pandas(pd.read_table(url))
    elif "tsv.bz2" in filetype:
        return pl.from_pandas(pd.read_csv(url, sep="\t", index_col=None), include_index=False)
    else:
        return pl.read_csv(url, separator="\t")



import unittest
from unittest.mock import patch, call
from progenomes import download

class TestDownload(unittest.TestCase):

    def test_get_genome_url(self):
        # Test case 1: representative-genomes
        url = download._get_genome_url('representative-genomes', 'proteins')
        expected_url = 'https://progenomes.embl.de/data/repGenomes/progenomes3.proteins.representatives.fasta.bz2'
        self.assertEqual(url, expected_url)

        # Test case 2: aquatic
        url = download._get_genome_url('aquatic', 'contigs')
        expected_url = 'https://progenomes.embl.de/data/habitats/representatives.aquatic.contigs.fasta.gz'
        self.assertEqual(url, expected_url)

    @patch('urllib.request.urlretrieve')
    def test_download_genomes(self, mock_urlretrieve):
        components = ['proteins', 'contigs']
        download.download_genomes('representative-genomes', components)

        # Check that urlretrieve was called with the correct URLs
        expected_calls = [
            call('https://progenomes.embl.de/data/repGenomes/progenomes3.proteins.representatives.fasta.bz2', 'progenomes3.proteins.representatives.fasta.bz2'),
            call('https://progenomes.embl.de/data/repGenomes/progenomes3.contigs.representatives.fasta.bz2', 'progenomes3.contigs.representatives.fasta.bz2')
        ]
        mock_urlretrieve.assert_has_calls(expected_calls, any_order=True)

    def test_get_dataset_url(self):
        # Test case 1: habitats-per-isolate
        url = download._get_dataset_url('habitats-per-isolate')
        expected_url = 'https://progenomes.embl.de/data/proGenomes3_habitat_isolates.tab.bz2'
        self.assertEqual(url, expected_url)

        # Test case 2: highly-important-strains
        url = download._get_dataset_url('highly-important-strains')
        expected_url = 'https://progenomes.embl.de/data/highly_important_strains.tab.bz2'
        self.assertEqual(url, expected_url)

    @patch('urllib.request.urlretrieve')
    def test_download_dataset(self, mock_urlretrieve):
        download.download_dataset('habitats-per-isolate')
        mock_urlretrieve.assert_called_once_with(
            'https://progenomes.embl.de/data/proGenomes3_habitat_isolates.tab.bz2',
            'proGenomes3_habitat_isolates.tab.bz2'
        )

if __name__ == '__main__':
    unittest.main()



import unittest
from unittest.mock import patch, MagicMock
from progenomes import view

class TestView(unittest.TestCase):

    def test_get_url(self):
        # Test with a valid item
        url, filetype = view.get_url('habitats-per-isolate')
        self.assertEqual(url, 'https://progenomes.embl.de/data/proGenomes3_habitat_isolates.tab.bz2')
        self.assertEqual(filetype, 'tab.bz2')

        # Test with another valid item
        url, filetype = view.get_url('highly-important-strains')
        self.assertEqual(url, 'https://progenomes.embl.de/data/highly_important_strains.tab.bz2')
        self.assertEqual(filetype, 'tab.bz2')

        # Test with an invalid item
        with self.assertRaises(ValueError):
            view.get_url('invalid-item')

    @patch('polars.from_pandas')
    @patch('pandas.read_table')
    def test_view_tab_bz2(self, mock_read_table, mock_from_pandas):
        # Mock the return value of get_url
        with patch('progenomes.view.get_url', return_value=('some_url.tab.bz2', 'tab.bz2')):
            view.view('any_target')
            mock_read_table.assert_called_once_with('some_url.tab.bz2')
            mock_from_pandas.assert_called_once()

    @patch('polars.from_pandas')
    @patch('pandas.read_csv')
    def test_view_tsv_bz2(self, mock_read_csv, mock_from_pandas):
        # Mock the return value of get_url
        with patch('progenomes.view.get_url', return_value=('some_url.tsv.bz2', 'tsv.bz2')):
            view.view('any_target')
            mock_read_csv.assert_called_once_with('some_url.tsv.bz2', sep='\t', index_col=None)
            mock_from_pandas.assert_called_once()

    @patch('polars.read_csv')
    def test_view_other(self, mock_read_csv):
        # Mock the return value of get_url
        with patch('progenomes.view.get_url', return_value=('some_url.csv', 'csv')):
            view.view('any_target')
            mock_read_csv.assert_called_once_with('some_url.csv', separator='\t')


if __name__ == '__main__':
    unittest.main()



from collections import namedtuple
from tqdm import tqdm
import urllib.request
from typing import Union

GENOME_INITIAL_URL = "https://progenomes.embl.de/data"

UrlData = namedtuple(
    "UrlData", ["type", "server_prefix", "file_prefix", "filetype", "order"]
)

DatasetUrlData = namedtuple(
    "DatasetUrlData", ["file_prefix", "filename", "filetype", "headers"]
)

GENOME_URL_MAPPING = {
    "representative-genomes": UrlData(
        "representatives", "repGenomes", "progenomes3", "fasta.bz2", "target.type"
    ),
    "aquatic": UrlData(
        "aquatic", "habitats", "representatives", "fasta.gz", "type.target"
    ),
    "disease-associated": UrlData(
        "disease-associated",
        "habitats",
        "representatives",
        "fasta.gz",
        "type.target",
    ),
    "food-associated": UrlData(
        "food-associated", "habitats", "representatives", "fasta.gz", "type.target"
    ),
    "freshwater": UrlData(
        "freshwater", "habitats", "representatives", "fasta.gz", "type.target"
    ),
    "host-associated": UrlData(
        "host-associated", "habitats", "representatives", "fasta.gz", "type.target"
    ),
    "host-plant-associated": UrlData(
        "host-plant-associated",
        "habitats",
        "representatives",
        "fasta.gz",
        "type.target",
    ),
    "sediment-mud": UrlData(
        "sediment-mud", "habitats", "representatives", "fasta.gz", "type.target"
    ),
    "soil": UrlData("soil", "habitats", "representatives", "fasta.gz", "type.target"),
}


def _get_genome_url(target: str, component: str):
    mapping = GENOME_URL_MAPPING[target]
    if mapping.order == "type.target":
        return (
            f"{GENOME_INITIAL_URL}/{mapping.server_prefix}/{mapping.file_prefix}."
            f"{mapping.type.replace('-', '_')}.{component}.{mapping.filetype}"
        )
    else:
        return (
            f"{GENOME_INITIAL_URL}/{mapping.server_prefix}/{mapping.file_prefix}."
            f"{component}.{mapping.type.replace('-', '_')}.{mapping.filetype}"
        )


def download_genomes(target: str, components: list[str]):
    ''' Download genome files by their target name and components. '''
    pbar = tqdm(components)
    for t in pbar:
        pbar.set_description(f"Downloading {t}")
        url = _get_genome_url(target, t)
        urllib.request.urlretrieve(url, url.split("/")[-1])


DATASET_INITIAL_URL = "https://progenomes.embl.de/data"

DATASET_URL_MAPPING = {
    "habitats-per-isolate": DatasetUrlData(
        "proGenomes3", "habitat_isolates", "tab.bz2", True
    ),
    "habitats-per-speci-cluster": DatasetUrlData(
        "proGenomes3", "habitat_specI", "tab.bz2", True
    ),
    "representatives-per-speci-cluster": DatasetUrlData(
        "proGenomes3", "representatives_for_each_specI", "tsv.gz", False
    ),
    "marker-genes": DatasetUrlData("proGenomes3", "markerGenes", "tar.gz", None),
    "speci-clustering-data": DatasetUrlData(
        "proGenomes3", "specI_clustering", "tab.bz2", False
    ),
    "gtdb-taxonomy": DatasetUrlData(
        "proGenomes3", "specI_lineageGTDB", "tab.bz2", True
    ),
    "highly-important-strains": DatasetUrlData(
        None, "highly_important_strains", "tab.bz2", False
    ),
    "excluded-genomes": DatasetUrlData(
        "proGenomes3", "excluded_genomes", "txt.bz2", None
    ),
    "mge-orfs": DatasetUrlData("representatives", "mge_ORFS", "tsv.bz2", True),
    "mge-annotation": DatasetUrlData(
        "representatives", "mge_annotation", "tsv.bz2", True
    ),
    "gecco-gene-clusters": DatasetUrlData(
        "proGenomes3", "gecco_clusters", "gbk.gz", None
    ),
}


def _get_dataset_url(item: str):
    mapping = DATASET_URL_MAPPING[item]
    if mapping.file_prefix is None:
        return f"{DATASET_INITIAL_URL}/{mapping.filename}.{mapping.filetype}"
    else:
        return (
            f"{DATASET_INITIAL_URL}/{mapping.file_prefix}_{mapping.filename}."
            f"{mapping.filetype}"
        )


def download_dataset(target: str):
    ''' Download a dataset by its target name. '''
    url = _get_dataset_url(target)
    urllib.request.urlretrieve(url, url.split("/")[-1])



from progenomes.download import download_dataset, download_genomes
from progenomes.view import view
import argparse


def main():
    parser = argparse.ArgumentParser(
        description="""
    Interact with the proGenomes[1] database.

    Anthony Fullam, Ivica Letunic, Thomas S B Schmidt, Quinten R Ducarmon,
    Nicolai Karcher, Supriya Khedkar, Michael Kuhn, Martin Larralde, Oleksandr
    M Maistrenko, Lukas Malfertheiner, Alessio Milanese, Joao Frederico Matias
    Rodrigues, Claudia Sanchis-López, Christian Schudoma, Damian Szklarczyk,
    Shinichi Sunagawa, Georg Zeller, Jaime Huerta-Cepas, Christian von Mering,
    Peer Bork, Daniel R Mende, proGenomes3: approaching one million accurately
    and consistently annotated high-quality prokaryotic genomes, Nucleic
    Acids Research, Volume 51, Issue D1, 6 January 2023, Pages D760–D766,
    https://doi.org/10.1093/nar/gkac1078
    """,
        formatter_class=argparse.RawTextHelpFormatter,
    )
    subparsers = parser.add_subparsers(help="subcommand help", dest="action")
    parser_download = subparsers.add_parser(
        "download", help="download data from an item"
    )
    subparser_download = parser_download.add_subparsers(help="subsubcommand help", dest="download_target")
    parser_download_genome = subparser_download.add_parser("genomes", help="download genome set")
    parser_download_genome.add_argument(
        dest="target",
        choices=[
            "representative-genomes",
            "aquatic",
            "disease-associated",
            "food-associated",
            "host-associated",
            "host-plant-associated",
            "freshwater",
            "sediment-mud",
            "soil",
        ],
        action="store",
        help="""Representative genome set to download. Available options:
        Representative genomes, Aquatic, Disease associated, Food associated,
        Freshwater, Host associated, Host plant associated, Sediment mud, Soil.""",
    )
    parser_download_genome.add_argument(
        "-c", "--contigs", dest="contigs", help="Contigs", action="store_true"
    )
    parser_download_genome.add_argument(
        "-g", "--genes", dest="genes", help="Genes", action="store_true"
    )
    parser_download_genome.add_argument(
        "-p", "--proteins", dest="proteins", help="Proteins", action="store_true"
    )
    parser_download_genome.add_argument(
        "-a", "--all", dest="all", help="All", action="store_true"
    )
    parser_download_dataset = subparser_download.add_parser("datasets", help="download genome set")
    parser_download_dataset.add_argument(
        dest="target",
        choices=[
            "habitats-per-isolate",
            "habitats-per-speci-cluster",
            "representatives-per-speci-cluster",
            "marker-genes",
            "speci-clustering-data",
            "gtdb-taxonomy",
            "highly-important-strains",
            "excluded-genomes",
            "mge-orfs",
            "mge-annotation",
            "gecco-gene-clusters",
        ],
        action="store",
        help="""Dataset to download. Available options:
        Habitats per isolate, Habitats per specI cluster, Representatives per specI cluster,
        Marker genes, SpecI clustering data, GTDB taxonomy, Highly important strains,
        Excluded genomes, MGE ORFs, MGE annotation, GECCO biosynthetic gene clusters (GenBank records)""",
    )
    parser_view = subparsers.add_parser("view", help="view an item")
    parser_view.add_argument(
        dest="target",
        choices=[
            "habitats-per-isolate",
            "habitats-per-speci-cluster",
            "representatives-per-speci-cluster",
            "speci-clustering-data",
            "gtdb-taxonomy",
            "highly-important-strains",
            "mge-orfs",
            "mge-annotation",
        ],
        action="store",
        help="""Dataset to view. Available options:
        Habitats per isolate, Habitats per specI cluster, Representatives per specI cluster,
        SpecI clustering data, GTDB taxonomy, Highly important strains,
        MGE ORFs, MGE annotation""",
    )

    args = parser.parse_args()
    if args.action == "download":
        items_to_download = []
        if args.download_target == "genomes":
            if not args.contigs and not args.genes and not args.proteins:
                args.all = True
            if args.contigs:
                items_to_download.append("contigs")
            if args.genes:
                items_to_download.append("genes")
            if args.proteins:
                items_to_download.append("proteins")
            if args.all:
                items_to_download = ["genes", "contigs", "proteins"]
            download_genomes(args.target, items_to_download)
        else:
            items_to_download = None
            download_dataset(args.target)
    elif args.action == "view":
        item_to_view = args.target
        print(view(item_to_view))


if __name__ == "__main__":
    main()



import unittest
from unittest.mock import patch
import sys
import os
import http.server
import socketserver
import threading
import time
from progenomes.cli import progenomes
from io import StringIO

PORT = 8000
BASE_URL = f"http://localhost:{PORT}"
FIXTURES_DIR = os.path.join(os.path.dirname(__file__), '..', 'fixtures')

class Handler(http.server.SimpleHTTPRequestHandler):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, directory=FIXTURES_DIR, **kwargs)

class TestCliIntegration(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.httpd = socketserver.TCPServer(("", PORT), Handler)
        cls.server_thread = threading.Thread(target=cls.httpd.serve_forever)
        cls.server_thread.daemon = True
        cls.server_thread.start()
        time.sleep(1) # Give the server a moment to start

    @classmethod
    def tearDownClass(cls):
        cls.httpd.shutdown()
        cls.httpd.server_close()
        cls.server_thread.join()

    def tearDown(self):
        # Clean up downloaded files
        if os.path.exists('proGenomes3_habitat_isolates.tab.bz2'):
            os.remove('proGenomes3_habitat_isolates.tab.bz2')

    @patch('progenomes.download.DATASET_INITIAL_URL', BASE_URL)
    def test_download_dataset_integration(self):
        sys.argv = ['progenomes', 'download', 'datasets', 'habitats-per-isolate']
        progenomes.main()
        self.assertTrue(os.path.exists('proGenomes3_habitat_isolates.tab.bz2'))
        # Further checks could be added here to verify file content

    @patch('progenomes.view.INITIAL_URL', BASE_URL)
    def test_view_integration(self):
        # This test will fetch the file from the local server
        sys.argv = ['progenomes', 'view', 'habitats-per-isolate']
        with patch('sys.stdout', new_callable=StringIO) as mock_stdout:
            progenomes.main()
            output = mock_stdout.getvalue()
            self.assertIn('col1', output)
            self.assertIn('val1', output)

if __name__ == '__main__':
    unittest.main()
